#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <usb.h>

#include "dcdcusbconst.h"

int dcdc_send ( unsigned char *data, int size, usb_dev_handle *dev) {
	if (data == NULL)
		return -1;

	int error = usb_interrupt_write (dev, USB_ENDPOINT_OUT + 1, (char *) data, size, 1000);
	if( error != size)
		fprintf(stderr, "error on usb_interrupt_write, returned code %d\n", error);

	return error;

}

int dcdc_receive ( unsigned char *data, int length, int timeout, usb_dev_handle *dev) {
	if (data == NULL)
		return -1;

	int error = usb_interrupt_read (dev, USB_ENDPOINT_IN + 1, (char *) data, length, timeout);
	if( error != length)
		fprintf(stderr, "error on usb_interrupt_read, returned code %d\n", error);

	return error;

}

int dcdc_get_all_values(usb_dev_handle *dev) {
	unsigned char packet[2];

	packet[0] = DCDCUSB_GET_ALL_VALUES;
	packet[1] = 0;

	return dcdc_send (packet, 2, dev);
}

int dcdc_read_status (uint8_t * data, usb_dev_handle *dev) {
	unsigned char packet[24];
	int len, i;
	int error;

	len = sizeof(packet);
	memset (packet, 0, len);
	//packet[0] = 0x82;
	error = dcdc_receive (packet, len, 1000, dev);
	if (error < 0) {
		fprintf(stderr, "receive failed");
		return -1;
	}

	//fprintf(stderr, "length=%d\n", error);
	for (i = 0; i < len; i++) {
		data[i] = packet[i];
	}

	return error;

}

usb_dev_handle *dcdc_open() {
	usb_init ();
	usb_set_debug(0);
	usb_find_busses();
	usb_find_devices();


	usb_dev_handle *dev;
	struct usb_bus *bus;
	for (bus = usb_get_busses (); bus != NULL; bus = bus->next) {
		struct usb_device *tmp_dev;

		for (tmp_dev = bus->devices; tmp_dev != NULL; tmp_dev = tmp_dev->next) {
			if ((tmp_dev->descriptor.idVendor ==  0x04d8) &&
				(tmp_dev->descriptor.idProduct == 0xd003)) {
				printf("Found DCDC-USB device\n");

				dev = usb_open (tmp_dev);
				break;
			}
		}
	}

	if (NULL == dev) {
		fprintf(stderr, "Couldn't find DCDC-USB device\n");
		return NULL;
	}

	char driver[1024];
	if (usb_get_driver_np (dev, 0, driver, sizeof (driver)) == 0) {
		fprintf(stderr,"Interface already claimed by '%s' detaching\n", driver);
		if ((usb_detach_kernel_driver_np (dev, 0) < 0)) {
			fprintf(stderr, "unable to detach %s driver", driver);
			usb_close (dev);
			return NULL;
		}
	}


	usb_reset(dev);

	int error = 0;
	error = usb_set_configuration (dev, 1);
	if(error != 0) {
		fprintf(stderr, " unable to set configuration: %d\n", error);
		usb_close (dev);
		return NULL;
	}

	if ((error = usb_claim_interface (dev, 0)) < 0) {
		fprintf(stderr, "Cannot claim interface ! error=%d\n", error);
		usb_close (dev);
		return NULL;
	}

        usleep (100);
        if ((error = usb_set_altinterface (dev, 0)) < 0) {
		fprintf(stderr, "unable to set alternate configuration: %d\n", error);
		usb_close (dev);
		return NULL;
	}

	char buf[65535];
	error = usb_control_msg(dev, USB_TYPE_CLASS + USB_RECIP_INTERFACE, 0x000000a, 0x0000000, 0x0000000, buf, 0x0000000, 1000);
	if(error != 0) {
		fprintf(stderr, "unable to send control message: %d", error );
		usb_close (dev);
		return NULL;
	}

	return dev;

}

void dcdc_close(usb_dev_handle *dev) {
	usb_release_interface (dev, 0);
	usb_close (dev);
}



